@@ -22,10 +22,6 @@
 #include <mednafen/Time.h>
 #include <trio/trio.h>
 
-#include <math.h>
-
-#include <vector>
-
 namespace MDFN_IEN_DEMO
 {
 
@@ -112,12 +108,28 @@ static void Draw(EmulateSpecStruct* espec, const uint8* weak, const uint8* stron
 
  if(!espec->skip)
  {
-  if(cur_test_mode == 2 || cur_test_mode == 3 || cur_test_mode == 4)
+  if(cur_test_mode == 5)
+   espec->surface->Fill(0xFF, 0xFF, 0xFF, 0);
+  else if(cur_test_mode == 2 || cur_test_mode == 3 || cur_test_mode == 4)
    espec->surface->Fill(0, 0, 0, 0);
   else
    espec->surface->Fill(DemoRandU32() & 0xFF, DemoRandU32() & 0xFF, DemoRandU32() & 0xFF, 0);
 
-  if(cur_test_mode == 4)
+  if(cur_test_mode == 5)
+  {
+   espec->DisplayRect.x = 0;
+   espec->DisplayRect.y = 0;
+   espec->DisplayRect.h = 300 << Interlace;
+   espec->DisplayRect.w = 0;
+
+   espec->LineWidths[0] = 0;
+
+   static const int wtab[4] = { 330, 352, 660, 704 };
+
+   for(int y = espec->DisplayRect.y; y < (espec->DisplayRect.y + espec->DisplayRect.h); y++)
+    espec->LineWidths[y] = wtab[DemoRandU32() & 0x3];
+  }
+  else if(cur_test_mode == 4)
   {
    espec->DisplayRect.x = 0;
    espec->DisplayRect.y = 0;
@@ -282,11 +294,16 @@ static void Emulate(EmulateSpecStruct* espec)
   {
    uint8 cur_cstate = *controller_ptr[i];
 
-   if((cur_cstate ^ last_cstate[i]) & cur_cstate & 1)
+   if(cur_test_mode == 5)
+   {
+    if(cur_cstate & 1)
+     Interlace = DemoRandU32() & 1;
+   }
+   else if((cur_cstate ^ last_cstate[i]) & cur_cstate & 1)
     Interlace = !Interlace;
 
    if((cur_cstate ^ last_cstate[i]) & cur_cstate & 2)
-    cur_test_mode = (cur_test_mode + 1) % 5;
+    cur_test_mode = (cur_test_mode + 1) % 6;
 
    last_cstate[i] = cur_cstate;
   }
@@ -370,6 +387,15 @@ static void Load(MDFNFILE* fp)
 {
  try
  {
+  std::vector<uint64> dlist = MDFN_GetSettingMultiUI("demo.multi_enum");
+
+  MDFN_printf("Loaded desserts:");
+
+  for(uint64 dle : dlist)
+   MDFN_printf(" %llu", (unsigned long long)dle);
+
+  MDFN_printf("\n");
+
   for(unsigned ch = 0; ch < 2; ch++)
    HRBufs[ch] = new OwlBuffer();
 
@@ -449,20 +475,33 @@ static void DoSimpleCommand(int cmd)
  }
 }
 
-static MDFNSetting DEMOSettings[] = 
+static const MDFNSetting_EnumList MultiEnum_List[] =
+{
+ { "cookie", 0, gettext_noop("Monster.") },
+ { "cake", 1, gettext_noop("Lie.") },
+ { "cobbler", 2, gettext_noop("Shoe.") },
+ { "strudel", 3, gettext_noop("Steve.") },
+
+ { NULL, 0 },
+};
+
+static const MDFNSetting DEMOSettings[] = 
 {
  { "demo.resamp_quality", MDFNSF_NOFLAGS, gettext_noop("Sound quality."), gettext_noop("Higher values correspond to better SNR and better preservation of higher frequencies(\"brightness\"), at the cost of increased computational complexity and a negligible increase in latency.\n\nHigher values will also slightly increase the probability of sample clipping(relevant if Mednafen's volume control settings are set too high), due to increased (time-domain) ringing."), MDFNST_INT, "3", "0", "5" },
  { "demo.resamp_rate_error", MDFNSF_NOFLAGS, gettext_noop("Sound output rate tolerance."), gettext_noop("Lower values correspond to better matching of the output rate of the resampler to the actual desired output rate, at the expense of increased RAM usage and poorer CPU cache utilization."), MDFNST_FLOAT, "0.0000009", "0.0000001", "0.0000350" },
+
+ { "demo.multi_enum", MDFNSF_NOFLAGS, gettext_noop("Multi-enum test."), nullptr, MDFNST_MULTI_ENUM, "", nullptr, nullptr, nullptr, nullptr, MultiEnum_List },
+
  { NULL }
 };
 
-static const char* SwitchPositions[] =
+static const IDIIS_SwitchPos SwitchPositions[] =
 {
- gettext_noop("Waffles 0"),
- gettext_noop("Oranges 1"),
- gettext_noop("Monkeys 2"),
- gettext_noop("Zebra-Z 3"),
- gettext_noop("Snorkle 4")
+ { "waffles", gettext_noop("Waffles 0"), gettext_noop("With extra churned ungulate squeezings and concentrated tree blood.") },
+ { "oranges", gettext_noop("Oranges 1"), gettext_noop("Blood oranges, of course.") },
+ { "monkeys", gettext_noop("Monkeys 2"), gettext_noop("Quiet, well-behaved monkeys that don't try to eat your face.") },
+ { "zebraz", gettext_noop("Zebra-Z 3"), gettext_noop("Zebras wearing sunglasses.") },
+ { "snorkle", gettext_noop("Snorkle 4"), gettext_noop("? ? ?") },
 };
 
 static const IDIISG IDII =
