@@ -20,8 +20,7 @@
 #include <trio/trio.h>
 #include "console.h"
 #include "video.h"
-#include <mednafen/string/trim.h>
-#include <vector>
+#include <mednafen/string/string.h>
 
 static MDFN_Thread *CheatThread = NULL;
 static MDFN_Mutex *CheatMutex = NULL;
