@@ -17,7 +17,6 @@
 
 #include "main.h"
 
-#include <string.h>
 #include <ctype.h>
 #include <trio/trio.h>
 #include <map>
@@ -34,10 +33,18 @@
 #include "help.h"
 #include "rmdui.h"
 
-#include <math.h>
-
 extern JoystickManager *joy_manager;
 
+int NoWaiting = 0;
+
+static bool ViewDIPSwitches = false;
+static bool InputGrab = false;
+static bool EmuKeyboardKeysGrabbed = false;
+static bool NPCheatDebugKeysGrabbed = false;
+
+static bool inff = 0;
+static bool insf = 0;
+
 bool RewindState = true;
 bool DNeedRewind = false;
 
@@ -58,8 +65,8 @@ static int subcon(const char *text, std::vector<ButtConfig> &bc, int commandkey)
 
 static void ResyncGameInputSettings(unsigned port);
 
-static char keys[MKK_COUNT];
-static char keys_untouched[MKK_COUNT];
+static uint8 keys[MKK_COUNT];
+static uint8 keys_untouched[MKK_COUNT];	// == 0 if not pressed, != 0 if pressed
 
 static std::string BCToString(const ButtConfig &bc)
 {
@@ -224,7 +231,7 @@ struct ButtonInfoCache
  int BCPrettyPrio = 0;
 
  unsigned Flags = 0;
- uint16 BitOffsets[4] = { 0, 0, 0, 0 };	// [rotated]
+ uint16 BitOffset = 0;
  uint16 ExclusionBitOffset = 0xFFFF;
  InputDeviceInputType Type = IDIT_BUTTON;
  bool Rapid = false;
@@ -283,6 +290,9 @@ static void KillPortInfo(unsigned int port)
  PIDC[port].RIC.clear();
 }
 
+//
+// Only call on init, or when the device on a port changes.
+//
 static void BuildPortInfo(MDFNGI *gi, const unsigned int port)
 {
  const InputDeviceInfoStruct *zedevice = NULL;
@@ -355,11 +365,7 @@ static void BuildPortInfo(MDFNGI *gi, const unsigned int port)
      bic.SettingName = trio_aprintf("%s.input.%s.%s.%s%s", gi->shortname, gi->PortInfo[port].ShortName, zedevice->ShortName, (r ? "rapid_" : ""), idii.SettingName);
      CleanSettingName(bic.SettingName);
      bic.BCANDMode = StringToBC(MDFN_GetSettingS(bic.SettingName).c_str(), bic.BC);
-
-     for(unsigned o = 0; o < 4; o++)
-     {
-      bic.BitOffsets[o] = idii.BitOffset;
-     }
+     bic.BitOffset = idii.BitOffset;
 
      if(idii.Type == IDIT_SWITCH)
      {
@@ -437,7 +443,7 @@ static void BuildPortInfo(MDFNGI *gi, const unsigned int port)
   }
 
   //
-  // Now, search for exclusion buttons and rotated inputs.
+  // Now, search for exclusion buttons.
   //
   for(auto& bic : PIDC[port].BIC)
   {
@@ -447,29 +453,28 @@ static void BuildPortInfo(MDFNGI *gi, const unsigned int port)
     {
      if(!strcasecmp(bic.IDII->ExcludeName, sub_bic.IDII->SettingName))
      {
-      bic.ExclusionBitOffset = sub_bic.BitOffsets[0];
+      bic.ExclusionBitOffset = sub_bic.BitOffset;
       break;
      }
     }
    }
-
-   if(bic.IDII->RotateName[0])
-   {
-    for(auto const& sub_bic : PIDC[port].BIC)
-    {
-     for(int rodir = 0; rodir < 3; rodir++)
-     {
-      if(!strcasecmp(bic.IDII->RotateName[rodir], sub_bic.IDII->SettingName))
-      {
-       bic.BitOffsets[1 + rodir] = sub_bic.BitOffsets[0];
-       // No break for you!
-      }
-     }
-    }
-   }
   }
 
  PIDC[port].Data = MDFNI_SetInput(port, device);
+
+ //
+ // Set default switch positions.
+ //
+ for(auto& bic : PIDC[port].BIC)
+ {
+  if(bic.Type == IDIT_SWITCH)
+  {
+   char tmpsn[512];
+   trio_snprintf(tmpsn, sizeof(tmpsn), "%s.defpos", bic.SettingName);
+   bic.SwitchLastPos = MDFN_GetSettingUI(tmpsn);
+   BitsIntract(PIDC[port].Data, bic.BitOffset, bic.SwitchBitSize, bic.SwitchLastPos);
+  }
+ }
 }
 
 static void IncSelectedDevice(unsigned int port)
@@ -482,7 +487,11 @@ static void IncSelectedDevice(unsigned int port)
  {
   MDFN_DispMessage(_("Cannot change input device while state rewinding is active."));
  }
- else if(CurGame->PortInfo[port].DeviceInfo.size() > 1)
+ else if(port >= CurGame->PortInfo.size())
+  MDFN_DispMessage(_("Port %u does not exist."), port + 1);
+ else if(CurGame->PortInfo[port].DeviceInfo.size() <= 1)
+  MDFN_DispMessage(_("Port %u device not selectable."), port + 1);
+ else
  {
   char tmp_setting_name[512];
 
@@ -539,8 +548,7 @@ enum CommandKey
 	CK_INSERTEJECT_DISK,
 	CK_ACTIVATE_BARCODE,
 
-        CK_TOGGLE_GRAB_INPUT,
-	CK_TOGGLE_CDISABLE,
+        CK_TOGGLE_GRAB,
 	CK_INPUT_CONFIG1,
 	CK_INPUT_CONFIG2,
         CK_INPUT_CONFIG3,
@@ -649,8 +657,7 @@ static const COKE CKeys[_CK_COUNT]	=
 	{ MK_CK(F6), "select_disk", false, 1, gettext_noop("Select disk/disc") },
 	{ MK_CK(F8), "insert_eject_disk", false, 0, gettext_noop("Insert/Eject disk/disc") },
 	{ MK_CK(F8), "activate_barcode", false, 1, gettext_noop("Activate barcode(for Famicom)") },
-	{ MK_CK(SCROLLOCK), "toggle_grab_input", false, 1, gettext_noop("Grab input") },
-	{ MK_CK_SHIFT(SCROLLOCK), "toggle_cidisable", false, 1, gettext_noop("Grab input and disable commands") },
+	{ MK_CK_CTRL_SHIFT(MENU), "toggle_grab", true, 1, gettext_noop("Grab input") },
 	{ MK_CK_ALT_SHIFT(1), "input_config1", false, 0, gettext_noop("Configure buttons on virtual port 1") },
 	{ MK_CK_ALT_SHIFT(2), "input_config2", false, 0, gettext_noop("Configure buttons on virtual port 2")  },
         { MK_CK_ALT_SHIFT(3), "input_config3", false, 0, gettext_noop("Configure buttons on virtual port 3")  },
@@ -727,17 +734,22 @@ static void CK_Init(void)
 
 static void CK_PostRemapUpdate(CommandKey which)
 {
- CKeysActive[which] = DTestButtonCombo(CKeysConfig[which].bc, (CKeys[which].BypassKeyZeroing ? keys_untouched : keys), MouseData, CKeysConfig[which].AND_Mode);
+ CKeysActive[which] = DTestButtonCombo(CKeysConfig[which].bc, ((CKeys[which].BypassKeyZeroing && (!EmuKeyboardKeysGrabbed || which == CK_TOGGLE_GRAB)) ? keys_untouched : keys), MouseData, CKeysConfig[which].AND_Mode);
  CKeysTrigger[which] = false;
  CKeysPressTime[which] = 0xFFFFFFFF;
 }
 
+static void CK_ClearTriggers(void)
+{
+ memset(CKeysTrigger, 0, sizeof(CKeysTrigger));
+}
+
 static void CK_UpdateState(bool skipckd_tc)
 {
  for(CommandKey i = _CK_FIRST; i < _CK_COUNT; i = (CommandKey)((unsigned)i + 1))
  {
   const bool prev_state = CKeysActive[i];
-  const bool cur_state = DTestButtonCombo(CKeysConfig[i].bc, (CKeys[i].BypassKeyZeroing ? keys_untouched : keys), MouseData, CKeysConfig[i].AND_Mode);
+  const bool cur_state = DTestButtonCombo(CKeysConfig[i].bc, ((CKeys[i].BypassKeyZeroing && (!EmuKeyboardKeysGrabbed || i == CK_TOGGLE_GRAB)) ? keys_untouched : keys), MouseData, CKeysConfig[i].AND_Mode);
   unsigned tmp_ckdelay = ckdelay;
 
   if(CKeys[i].SkipCKDelay || skipckd_tc)
@@ -751,8 +763,6 @@ static void CK_UpdateState(bool skipckd_tc)
   else
    CKeysPressTime[i] = 0xFFFFFFFF;
 
-  CKeysTrigger[i] = false;
-
   if(CurTicks >= ((uint64)CKeysPressTime[i] + tmp_ckdelay))
   {
    CKeysTrigger[i] = true;
@@ -773,14 +783,6 @@ static INLINE bool CK_CheckActive(CommandKey which)
  return CKeysActive[which];
 }
 
-int NoWaiting = 0;
-
-static bool ViewDIPSwitches = false;
-static int cidisabled=0;
-
-static bool inff = 0;
-static bool insf = 0;
-
 typedef enum
 {
 	none,
@@ -823,10 +825,34 @@ void MainSetEventHook(int (*eh)(const SDL_Event *event))
  EventHook = eh;
 }
 
+
+// Can be called from MDFND_MidSync(), so be careful.
 void Input_Event(const SDL_Event *event)
 {
  switch(event->type)
  {
+  case SDL_KEYDOWN:
+	{
+	 //printf("Down: %3u\n", event->key.keysym.sym);
+	 size_t s = event->key.keysym.sym;
+
+	 if(s < MKK_COUNT)
+	 {
+          keys_untouched[s] = (keys_untouched[s] & 0x7F) | 0x01;
+	 }
+	}
+	break;
+
+  case SDL_KEYUP:
+	{
+	 //printf("Up: %3u\n", event->key.keysym.sym);
+	 size_t s = event->key.keysym.sym;
+
+	 if(s < MKK_COUNT)
+	  keys_untouched[s] |= 0x80;
+	}
+	break;
+
   case SDL_MOUSEBUTTONDOWN:
 	if(event->button.state == SDL_PRESSED)
 	{
@@ -859,6 +885,11 @@ void Input_Event(const SDL_Event *event)
  still register as pressed for 1 emulated frame, and without otherwise increasing the lag of a mouse button release(which
  is what the button_prevsent is for).
 */
+/*
+ Note: Don't call this function frivolously or any more than needed, or else the logic to prevent lost key and mouse button
+ presses won't work properly in regards to emulated input devices(has particular significance with the "Pause" key and the emulated
+ Saturn keyboard).
+*/
 static void UpdatePhysicalDeviceState(void)
 {
  const bool clearify_mdr = true;
@@ -888,9 +919,20 @@ static void UpdatePhysicalDeviceState(void)
  //
  //
  //
+ for(unsigned i = 0; i < MKK_COUNT; i++)
+ {
+  uint8 tmp = keys_untouched[i];
 
+  if((tmp & 0x80) && ((tmp & 0x02) || !(tmp & 0x01)))
+   tmp = 0;
 
- memcpy(keys_untouched, SDL_GetKeyState(0), MKK_COUNT);
+  tmp = (tmp & 0x81) | ((tmp << 1) & 0x02);
+
+  keys_untouched[i] = tmp;
+
+  //if(tmp)
+  // printf("%3u, 0x%02x\n", i, tmp);
+ }
  memcpy(keys, keys_untouched, MKK_COUNT);
 
  if(MDFNDHaveFocus || MDFN_GetSettingB("input.joystick.global_focus"))
@@ -944,41 +986,80 @@ static uint8 BarcodeWorldData[1 + 13];
 
 static void DoKeyStateZeroing(void)
 {
-  if(IConfig == none && !(cidisabled & 0x1))
+ EmuKeyboardKeysGrabbed = false;
+ NPCheatDebugKeysGrabbed = false;
+
+ if(IConfig == none)
+ {
+  if(Debugger_IsActive())
   {
-   if(Netplay_IsTextInput() || CheatIF_Active())
-   {
-    memset(keys, 0, sizeof(keys)); // This effectively disables keyboard input, but still
+   memset(keys, 0, sizeof(keys));
+   NPCheatDebugKeysGrabbed = true;
+
+   keys[SDLK_F1] = keys_untouched[SDLK_F1];
+   keys[SDLK_F2] = keys_untouched[SDLK_F2];
+   keys[SDLK_F3] = keys_untouched[SDLK_F3];
+   keys[SDLK_F4] = keys_untouched[SDLK_F4];
+   keys[SDLK_F5] = keys_untouched[SDLK_F5];
+   keys[SDLK_F6] = keys_untouched[SDLK_F6];
+   keys[SDLK_F7] = keys_untouched[SDLK_F7];
+   keys[SDLK_F8] = keys_untouched[SDLK_F8];
+   keys[SDLK_F9] = keys_untouched[SDLK_F9];
+   keys[SDLK_F10] = keys_untouched[SDLK_F10];
+   keys[SDLK_F11] = keys_untouched[SDLK_F11];
+   keys[SDLK_F12] = keys_untouched[SDLK_F12];
+   keys[SDLK_F13] = keys_untouched[SDLK_F13];
+   keys[SDLK_F14] = keys_untouched[SDLK_F14];
+   keys[SDLK_F15] = keys_untouched[SDLK_F15];
+  }
+  else if(Netplay_IsTextInput() || CheatIF_Active())
+  {
+   memset(keys, 0, sizeof(keys)); // This effectively disables keyboard input, but still
                                    // allows physical joystick input when in the chat mode.
-   }
-
-   if(Debugger_IsActive())
+   NPCheatDebugKeysGrabbed = true;
+  }
+  else if(InputGrab)
+  {
+   for(unsigned int x = 0; x < CurGame->PortInfo.size(); x++)
    {
-    memset(keys, 0, sizeof(keys));
+    if(!(PIDC[x].Device->Flags & InputDeviceInfoStruct::FLAG_KEYBOARD))
+     continue;
 
-    keys[SDLK_F1] = keys_untouched[SDLK_F1];
-    keys[SDLK_F2] = keys_untouched[SDLK_F2];
-    keys[SDLK_F3] = keys_untouched[SDLK_F3];
-    keys[SDLK_F4] = keys_untouched[SDLK_F4];
-    keys[SDLK_F5] = keys_untouched[SDLK_F5];
-    keys[SDLK_F6] = keys_untouched[SDLK_F6];
-    keys[SDLK_F7] = keys_untouched[SDLK_F7];
-    keys[SDLK_F8] = keys_untouched[SDLK_F8];
-    keys[SDLK_F9] = keys_untouched[SDLK_F9];
-    keys[SDLK_F10] = keys_untouched[SDLK_F10];
-    keys[SDLK_F11] = keys_untouched[SDLK_F11];
-    keys[SDLK_F12] = keys_untouched[SDLK_F12];
-    keys[SDLK_F13] = keys_untouched[SDLK_F13];
-    keys[SDLK_F14] = keys_untouched[SDLK_F14];
-    keys[SDLK_F15] = keys_untouched[SDLK_F15];
+    for(auto& bic : PIDC[x].BIC)
+    {
+     for(auto& b : bic.BC)
+     {
+      if(b.ButtType != BUTTC_KEYBOARD)
+       continue;
+
+      EmuKeyboardKeysGrabbed = true;
+
+#if 1
+      memset(keys, 0, sizeof(keys));
+      goto IGrabEnd;
+#else
+      //
+      if(b.ButtonNum & (2 << 24)) // Shift
+       keys[MKK(LSHIFT)] = keys[MKK(RSHIFT)] = 0;
+      if(b.ButtonNum & (1 << 24)) // Alt
+       keys[MKK(LALT)] = keys[MKK(RALT)] = 0;
+      if(b.ButtonNum & (4 << 24)) // Ctrl
+       keys[MKK(LCTRL)] = keys[MKK(RCTRL)] = 0;
+      //
+      unsigned k = b.ButtonNum & 0xFFFFFF;
+      if(k < MKK_COUNT)
+       keys[k] = 0;
+#endif
+     }
+    }
    }
+   IGrabEnd:;
+  }
  }
 }
 
 static void CheckCommandKeys(void)
 {
-  CK_UpdateState((IConfig == Command || IConfig == CommandAM) && ICLatch == -1);
-
   for(unsigned i = 0; i < 12; i++)
   {
    if(IConfig == Port1 + i)
@@ -1035,41 +1116,24 @@ static void CheckCommandKeys(void)
    }
   }
 
-  {
-   bool cid_changed = false;
-
-   if(CK_Check(CK_TOGGLE_GRAB_INPUT))
-   {
-    cidisabled = (cidisabled == 0x2) ? 0 : 0x2;
-    cid_changed = true;
-   }
-
-   if(CK_Check(CK_TOGGLE_CDISABLE))
-   {
-    cidisabled = (cidisabled == 0x3) ? 0 : 0x3;
-    cid_changed = true;
-   }
-
-   if(cid_changed)
-   {
-    SDL_Event evt;
-    evt.user.type = SDL_USEREVENT;
-    evt.user.code = CEVT_SET_GRAB_INPUT;
-    evt.user.data1 = malloc(1);
-
-    *(uint8 *)evt.user.data1 = cidisabled & 0x2;
-    SDL_PushEvent(&evt);
-
-    MDFNI_DispMessage(_("Input grabbing: %s, Command key processing: %s"), (cidisabled & 0x02) ? _("On") : _("Off"), !(cidisabled & 0x01) ? _("On") : _("Off"));
-   }
-  }
-
-  if(cidisabled & 0x1)
-   return;
-
   if(IConfig != none)
    return;
 
+  if(CK_Check(CK_TOGGLE_GRAB))
+  {
+   InputGrab = !InputGrab;
+   //
+   SDL_Event evt;
+   evt.user.type = SDL_USEREVENT;
+   evt.user.code = CEVT_SET_GRAB_INPUT;
+   evt.user.data1 = malloc(1);
+
+   *(uint8 *)evt.user.data1 = InputGrab;
+   SDL_PushEvent(&evt);
+
+   MDFNI_DispMessage(_("Input grabbing: %s"), InputGrab ? _("On") : _("Off"));
+  }
+
   if(!CheatIF_Active() && !MDFNDnetplay)
   {
    if(CK_Check(CK_TOGGLE_DEBUGGER))
@@ -1149,11 +1213,13 @@ static void CheckCommandKeys(void)
 
   if(!Debugger_IsActive()) // We don't want to start button configuration when the debugger is active!
   {
-   for(int i = 0; i < 12; i++)
+   for(unsigned i = 0; i < 12; i++)
    {
     if(CK_Check((CommandKey)(CK_INPUT_CONFIG1 + i)))
     {
-     if(!PIDC[i].BIC.size())
+     if(i >= CurGame->PortInfo.size())
+      MDFN_DispMessage(_("Port %u does not exist."), i + 1);
+     else if(!PIDC[i].BIC.size())
      {
       MDFN_DispMessage(_("No buttons to configure for input port %u!"), i + 1);
      }
@@ -1412,21 +1478,32 @@ void MDFND_UpdateInput(bool VirtualDevicesOnly, bool UpdateRapidFire)
  // CheckCommandKeys(), specifically MDFNI_LoadState(), should be called *before* we update the emulated device input data, as that data is 
  // stored/restored from save states(related: ALT+A frame advance, switch state).
  //
+ CK_UpdateState((IConfig == Command || IConfig == CommandAM) && ICLatch == -1);
  if(!VirtualDevicesOnly)
+ {
   CheckCommandKeys();
+  CK_ClearTriggers();
+ }
 
  if(UpdateRapidFire)
   rapid = (rapid + 1) % (autofirefreq + 1);
 
- bool RotateInput = false;
- bool RotateInputSettingFetched = false;
-
  // Do stuff here
  for(unsigned int x = 0; x < CurGame->PortInfo.size(); x++)
  {
+  uint8* kk = keys;
+
   if(!PIDC[x].Data)
    continue;
 
+  if(PIDC[x].Device->Flags & InputDeviceInfoStruct::FLAG_KEYBOARD)
+  {
+   if(!InputGrab || NPCheatDebugKeysGrabbed)
+    continue;
+   else
+    kk = keys_untouched;
+  }
+
   //
   // Handle rumble(FIXME: Do we want rumble to work in frame advance mode too?)
   //
@@ -1453,22 +1530,8 @@ void MDFND_UpdateInput(bool VirtualDevicesOnly, bool UpdateRapidFire)
   //
   for(auto& bic : PIDC[x].BIC)
   {
-   if(!RotateInputSettingFetched)
-   {
-    if(bic.BitOffsets[CurGame->rotated] != bic.BitOffsets[0])
-    {
-     char tmp_setting_name[512];
-
-     trio_snprintf(tmp_setting_name, 512, "%s.rotateinput", CurGame->shortname);
-     RotateInput = MDFN_GetSettingB(tmp_setting_name);
-     RotateInputSettingFetched = true;
-    }
-   }
-   //
-   //
-   //
    uint8* tptr = PIDC[x].Data;
-   const uint32 bo = bic.BitOffsets[RotateInput ? CurGame->rotated : 0];
+   const uint32 bo = bic.BitOffset;
    uint8* const btptr = &tptr[bo >> 3];
 
 
@@ -1480,7 +1543,7 @@ void MDFND_UpdateInput(bool VirtualDevicesOnly, bool UpdateRapidFire)
     float tv = 0;
 
     if(bic.BC.size() > 0)
-     tv = DTestMouseAxis(bic.BC[0], keys, MouseData, (bic.Type == IDIT_Y_AXIS));
+     tv = DTestMouseAxis(bic.BC[0], kk, MouseData, (bic.Type == IDIT_Y_AXIS));
 
     if(bic.Type == IDIT_Y_AXIS)
      tv = floor(0.5 + (tv * (1.0 / 65536) * CurGame->mouse_scale_y) + CurGame->mouse_offs_y);
@@ -1493,7 +1556,7 @@ void MDFND_UpdateInput(bool VirtualDevicesOnly, bool UpdateRapidFire)
    {
     uint32 intv;
 
-    intv = DTestAnalogButton(bic.BC, keys, MouseData);
+    intv = DTestAnalogButton(bic.BC, kk, MouseData);
  
     if(bic.Flags & IDIT_BUTTON_ANALOG_FLAG_SQLR)
      intv = std::min<unsigned>((unsigned)floor(0.5 + intv * PIDC[x].AxisScale), 32767);
@@ -1502,7 +1565,7 @@ void MDFND_UpdateInput(bool VirtualDevicesOnly, bool UpdateRapidFire)
    }
    else if(bic.Type == IDIT_SWITCH)
    {
-    const bool nps = DTestButton(bic.BC, keys, MouseData, bic.BCANDMode);
+    const bool nps = DTestButton(bic.BC, kk, MouseData, bic.BCANDMode);
     uint8 cv = BitsExtract(tptr, bo, bic.SwitchBitSize);
 
     if(MDFN_UNLIKELY(!bic.SwitchLastPress && nps))
@@ -1515,7 +1578,7 @@ void MDFND_UpdateInput(bool VirtualDevicesOnly, bool UpdateRapidFire)
      fprintf(stderr, "[BUG] cv(%u) >= bic.SwitchNumPos(%u)\n", cv, bic.SwitchNumPos);
     else if(MDFN_UNLIKELY(cv != bic.SwitchLastPos))
     {
-     MDFN_DispMessage(_("%s %u: %s: %s selected."), PIDC[x].Device->FullName, x + 1, bic.IDII->Name, bic.IDII->SwitchPosName[cv]);
+     MDFN_DispMessage(_("%s %u: %s: %s selected."), PIDC[x].Device->FullName, x + 1, bic.IDII->Name, bic.IDII->SwitchPos[cv].Name);
      bic.SwitchLastPos = cv;
     }
 
@@ -1526,7 +1589,7 @@ void MDFND_UpdateInput(bool VirtualDevicesOnly, bool UpdateRapidFire)
     if(!bic.Rapid)
      *btptr &= ~(1 << (bo & 7));
 
-    if(DTestButton(bic.BC, keys, MouseData, bic.BCANDMode)) // boolean button
+    if(DTestButton(bic.BC, kk, MouseData, bic.BCANDMode)) // boolean button
     {
      if(!bic.Rapid || rapid >= (autofirefreq + 1) / 2)
       *btptr |= 1 << (bo & 7);
@@ -1541,7 +1604,7 @@ void MDFND_UpdateInput(bool VirtualDevicesOnly, bool UpdateRapidFire)
   {
    if(bic.ExclusionBitOffset != 0xFFFF)
    {
-    const uint32 bo[2] = { bic.BitOffsets[0], bic.ExclusionBitOffset };
+    const uint32 bo[2] = { bic.BitOffset, bic.ExclusionBitOffset };
     const uint32 bob[2] = { bo[0] >> 3, bo[1] >> 3 };
     const uint32 bom[2] = { 1U << (bo[0] & 0x7), 1U << (bo[1] & 0x7) };
     uint8 *tptr = PIDC[x].Data;
@@ -1819,7 +1882,7 @@ static void MakeSettingsForDevice(std::vector <MDFNSetting> &settings, const MDF
     memset(&tmp_setting, 0, sizeof(tmp_setting));
 
     PendingGarbage.push_back((void *)( tmp_setting.name = CleanSettingName(trio_aprintf("%s.input.%s.%s.axis_scale", system->shortname, system->PortInfo[w].ShortName, info->ShortName)) ));
-    PendingGarbage.push_back((void *)( tmp_setting.description = trio_aprintf("Analog axis scale coefficient for %s on %s.", info->FullName, system->PortInfo[w].FullName) ));
+    PendingGarbage.push_back((void *)( tmp_setting.description = trio_aprintf(gettext_noop("Analog axis scale coefficient for %s on %s."), info->FullName, system->PortInfo[w].FullName) ));
     tmp_setting.description_extra = NULL;
 
     tmp_setting.type = MDFNST_FLOAT;
@@ -1831,6 +1894,38 @@ static void MakeSettingsForDevice(std::vector <MDFNSetting> &settings, const MDF
     analog_scale_made = true;
    }
   }
+  else if(info->IDII[x].Type == IDIT_SWITCH)
+  {
+   memset(&tmp_setting, 0, sizeof(tmp_setting));
+
+   PendingGarbage.push_back((void *)( tmp_setting.name = CleanSettingName(trio_aprintf("%s.input.%s.%s.%s.defpos", system->shortname, system->PortInfo[w].ShortName, info->ShortName, info->IDII[x].SettingName)) ));
+   PendingGarbage.push_back((void *)( tmp_setting.description = trio_aprintf(gettext_noop("Default position for switch \"%s\"."), info->IDII[x].Name) ));
+   tmp_setting.description_extra = gettext_noop("Sets the position for the switch to the value specified upon startup and virtual input device change.");
+
+   tmp_setting.flags = (info->IDII[x].Flags & IDIT_FLAG_AUX_SETTINGS_UNDOC) ? MDFNSF_SUPPRESS_DOC : 0;
+   {
+    MDFNSetting_EnumList* el = (MDFNSetting_EnumList*)calloc(info->IDII[x].SwitchNumPos + 1, sizeof(MDFNSetting_EnumList));
+    const uint32 snp = info->IDII[x].SwitchNumPos;
+
+    for(uint32 i = 0; i < snp; i++)
+    {
+     el[i].string = info->IDII[x].SwitchPos[i].SettingName;
+     el[i].number = i;
+     el[i].description = info->IDII[x].SwitchPos[i].Name;
+     el[i].description_extra = info->IDII[x].SwitchPos[i].Description;
+    }
+    el[snp].string = NULL;
+    el[snp].number = 0;
+    el[snp].description = NULL;
+    el[snp].description_extra = NULL;
+
+    PendingGarbage.push_back((void *)( tmp_setting.enum_list = el ));
+    tmp_setting.type = MDFNST_ENUM;
+    tmp_setting.default_value = el[0].string;
+   }
+
+   settings.push_back(tmp_setting);
+  }
   butti++;
  }
 }
@@ -1855,7 +1950,15 @@ static void MakeSettingsForPort(std::vector <MDFNSetting> &settings, const MDFNG
    EnumList[device].string = strdup(dinfo->ShortName);
    EnumList[device].number = device;
    EnumList[device].description = strdup(info->DeviceInfo[device].FullName);
-   EnumList[device].description_extra = info->DeviceInfo[device].Description ? strdup(info->DeviceInfo[device].Description) : NULL;
+
+   EnumList[device].description_extra = NULL;
+   if(info->DeviceInfo[device].Description || (info->DeviceInfo[device].Flags & InputDeviceInfoStruct::FLAG_KEYBOARD))
+   {
+    EnumList[device].description_extra = trio_aprintf("%s%s%s",
+	info->DeviceInfo[device].Description ? info->DeviceInfo[device].Description : "",
+	(info->DeviceInfo[device].Description && (info->DeviceInfo[device].Flags & InputDeviceInfoStruct::FLAG_KEYBOARD)) ? "\n" : "",
+	(info->DeviceInfo[device].Flags & InputDeviceInfoStruct::FLAG_KEYBOARD) ? _("Emulated keyboard key state is not updated unless input grabbing(by default, mapped to CTRL+SHIFT+Menu) is toggled on; refer to the main documentation for details.") : "");
+   }
 
    PendingGarbage.push_back((void *)EnumList[device].string);
    PendingGarbage.push_back((void *)EnumList[device].description);
