@@ -26,9 +26,6 @@
 #include <signal.h>
 #include <sys/time.h>
 #include <sys/stat.h>
-#include <string.h>
-#include <strings.h>
-#include <errno.h>
 #include <trio/trio.h>
 #include <locale.h>
 
@@ -40,7 +37,6 @@
 #include <iconv.h>
 #endif
 
-#include <algorithm>
 #include <atomic>
 
 #include "input.h"
@@ -61,7 +57,7 @@
 #include <mednafen/qtrecord.h>
 #include <mednafen/tests.h>
 #include <mednafen/MemoryStream.h>
-#include <math.h>
+#include <mednafen/file.h>
 
 static int StateSLSTest = false;
 
@@ -72,12 +68,13 @@ bool pending_save_state, pending_snapshot, pending_ssnapshot, pending_save_movie
 static uint32 volatile MainThreadID = 0;
 static bool ffnosound;
 
-static MDFNSetting_EnumList SDriver_List[] =
+static const MDFNSetting_EnumList SDriver_List[] =
 {
  { "default", -1, "Default", gettext_noop("Selects the default sound driver.") },
 
  { "alsa", -1, "ALSA", gettext_noop("The default for Linux(if available).") },
- { "oss", -1, "Open Sound System", gettext_noop("The default for non-Linux UN*X/POSIX/BSD systems, or anywhere ALSA is unavailable. If the ALSA driver gives you problems, you can try using this one instead.\n\nIf you are using OSSv4 or newer, you should edit \"/usr/lib/oss/conf/osscore.conf\", uncomment the max_intrate= line, and change the value from 100(default) to 1000(or higher if you know what you're doing), and restart OSS. Otherwise, performance will be poor, and the sound buffer size in Mednafen will be orders of magnitude larger than specified.\n\nIf the sound buffer size is still excessively larger than what is specified via the \"sound.buffer_time\" setting, you can try setting \"sound.period_time\" to 2666, and as a last resort, 5333, to work around a design flaw/limitation/choice in the OSS API and OSS implementation.") },
+ { "openbsd", -1, "OpenBSD Audio", gettext_noop("The default for OpenBSD.") },
+ { "oss", -1, "Open Sound System", gettext_noop("The default for non-Linux UN*X/POSIX/BSD(other than OpenBSD) systems, or anywhere ALSA is unavailable. If the ALSA driver gives you problems, you can try using this one instead.\n\nIf you are using OSSv4 or newer, you should edit \"/usr/lib/oss/conf/osscore.conf\", uncomment the max_intrate= line, and change the value from 100(default) to 1000(or higher if you know what you're doing), and restart OSS. Otherwise, performance will be poor, and the sound buffer size in Mednafen will be orders of magnitude larger than specified.\n\nIf the sound buffer size is still excessively larger than what is specified via the \"sound.buffer_time\" setting, you can try setting \"sound.period_time\" to 2666, and as a last resort, 5333, to work around a design flaw/limitation/choice in the OSS API and OSS implementation.") },
 
  { "wasapish", -1, "WASAPI(Shared Mode)", gettext_noop("The default when it's available(running on Microsoft Windows Vista and newer).") },
 
@@ -120,7 +117,7 @@ static const MDFNSetting_EnumList FontSize_List[] =
 
 
 static std::vector <MDFNSetting> NeoDriverSettings;
-static MDFNSetting DriverSettings[] =
+static const MDFNSetting DriverSettings[] =
 {
   { "input.joystick.global_focus", MDFNSF_NOFLAGS, gettext_noop("Update physical joystick(s) internal state in Mednafen even when Mednafen lacks OS focus."), NULL, MDFNST_BOOL, "1" },
   { "input.joystick.axis_threshold", MDFNSF_NOFLAGS, gettext_noop("Analog axis binary press detection threshold."), gettext_noop("Threshold for detecting a digital-like \"button\" press on analog axis, in percent."), MDFNST_FLOAT, "75", "0", "100" },
@@ -289,7 +286,7 @@ void MDFND_Message(const char *s)
 
 static void CreateDirs(void)
 {
- static const char *subs[] = { "mcs", "mcm", "snaps", "palettes", "sav", "cheats", "firmware", "pgconfig" };
+ static const char* const subs[] = { "mcs", "mcm", "snaps", "palettes", "sav", "cheats", "firmware", "pgconfig" };
 
  try
  {
@@ -462,56 +459,78 @@ static void Stream64Test(const char* path)
 {
  try
  {
- {
-  FileStream fp(path, FileStream::MODE_WRITE_SAFE);
+  {
+   FileStream fp(path, FileStream::MODE_WRITE_SAFE);
 
-  assert(fp.tell() == 0);
-  assert(fp.size() == 0);
-  fp.put_BE<uint32>(0xDEADBEEF);
-  assert(fp.tell() == 4);
-  assert(fp.size() == 4);
+   assert(fp.tell() == 0);
+   assert(fp.size() == 0);
+   fp.put_BE<uint32>(0xDEADBEEF);
+   assert(fp.tell() == 4);
+   assert(fp.size() == 4);
 
-  fp.seek((uint64)8192 * 1024 * 1024, SEEK_SET);
-  fp.put_BE<uint32>(0xCAFEBABE);
-  assert(fp.tell() == (uint64)8192 * 1024 * 1024 + 4);
-  assert(fp.size() == (uint64)8192 * 1024 * 1024 + 4);
+   fp.seek(0x7FFFFFFFU, SEEK_SET);
+   assert(fp.tell() == 0x7FFFFFFFU);
+   fp.truncate(0x7FFFFFFFU);
+   assert(fp.size() == 0x7FFFFFFFU);
+   fp.put_LE<uint8>(0xB0);
+   assert(fp.tell() == 0x80000000U);
+   assert(fp.size() == 0x80000000U);
+   fp.put_LE<uint8>(0x0F);
+   assert(fp.tell() == 0x80000001U);
+   assert(fp.size() == 0x80000001U);
 
-  fp.put_BE<uint32>(0xAAAAAAAA);
-  assert(fp.tell() == (uint64)8192 * 1024 * 1024 + 8);
-  assert(fp.size() == (uint64)8192 * 1024 * 1024 + 8);
+   fp.seek(0xFFFFFFFFU, SEEK_SET);
+   assert(fp.tell() == 0xFFFFFFFFU);
+   fp.truncate(0xFFFFFFFFU);
+   assert(fp.size() == 0xFFFFFFFFU);
+   fp.put_LE<uint8>(0xCA);
+   assert(fp.tell() == 0x100000000ULL);
+   assert(fp.size() == 0x100000000ULL);
+   fp.put_LE<uint8>(0xAD);
+   assert(fp.tell() == 0x100000001ULL);
+   assert(fp.size() == 0x100000001ULL);
 
-  fp.truncate((uint64)8192 * 1024 * 1024 + 4);
-  assert(fp.size() == (uint64)8192 * 1024 * 1024 + 4);
+   fp.seek((uint64)8192 * 1024 * 1024, SEEK_SET);
+   fp.put_BE<uint32>(0xCAFEBABE);
+   assert(fp.tell() == (uint64)8192 * 1024 * 1024 + 4);
+   assert(fp.size() == (uint64)8192 * 1024 * 1024 + 4);
 
-  fp.seek(-((uint64)8192 * 1024 * 1024 + 8), SEEK_CUR);
-  assert(fp.tell() == 0);
-  fp.seek((uint64)-4, SEEK_END);
-  assert(fp.tell() == (uint64)8192 * 1024 * 1024);
- }
+   fp.put_BE<uint32>(0xAAAAAAAA);
+   assert(fp.tell() == (uint64)8192 * 1024 * 1024 + 8);
+   assert(fp.size() == (uint64)8192 * 1024 * 1024 + 8);
 
- {
-  FileStream fp(path, FileStream::MODE_READ);
-  uint32 tmp;
+   fp.truncate((uint64)8192 * 1024 * 1024 + 4);
+   assert(fp.size() == (uint64)8192 * 1024 * 1024 + 4);
 
-  assert(fp.size() == (uint64)8192 * 1024 * 1024 + 4);
-  tmp = fp.get_LE<uint32>();
-  assert(tmp == 0xEFBEADDE);
-  fp.seek((uint64)8192 * 1024 * 1024 - 4, SEEK_CUR);
-  tmp = fp.get_LE<uint32>(); 
-  assert(tmp == 0xBEBAFECA);
- }
+   fp.seek(-((uint64)8192 * 1024 * 1024 + 8), SEEK_CUR);
+   assert(fp.tell() == 0);
+   fp.seek((uint64)-4, SEEK_END);
+   assert(fp.tell() == (uint64)8192 * 1024 * 1024);
+  }
 
- {
-  GZFileStream fp(path, GZFileStream::MODE::READ);
-  uint32 tmp;
+  {
+   FileStream fp(path, FileStream::MODE_READ);
+   uint32 tmp;
 
-  tmp = fp.get_LE<uint32>();
-  assert(tmp == 0xEFBEADDE);
-  fp.seek((uint64)8192 * 1024 * 1024 - 4, SEEK_CUR);
-  tmp = fp.get_LE<uint32>(); 
-  assert(tmp == 0xBEBAFECA);  
-  assert(fp.tell() == (uint64)8192 * 1024 * 1024 + 4);
- }
+   assert(fp.size() == (uint64)8192 * 1024 * 1024 + 4);
+   tmp = fp.get_LE<uint32>();
+   assert(tmp == 0xEFBEADDE);
+   fp.seek((uint64)8192 * 1024 * 1024 - 4, SEEK_CUR);
+   tmp = fp.get_LE<uint32>(); 
+   assert(tmp == 0xBEBAFECA);
+  }
+
+  {
+   GZFileStream fp(path, GZFileStream::MODE::READ);
+   uint32 tmp;
+
+   tmp = fp.get_LE<uint32>();
+   assert(tmp == 0xEFBEADDE);
+   fp.seek((uint64)8192 * 1024 * 1024 - 4, SEEK_CUR);
+   tmp = fp.get_LE<uint32>(); 
+   assert(tmp == 0xBEBAFECA);  
+   assert(fp.tell() == (uint64)8192 * 1024 * 1024 + 4);
+  }
  }
  catch(std::exception& e)
  {
@@ -613,8 +632,8 @@ static void DeleteInternalArgs(void)
 
 static void MakeMednafenArgsStruct(void)
 {
- const std::multimap <uint32, MDFNCS> *settings;
- std::multimap <uint32, MDFNCS>::const_iterator sit;
+ const std::vector<MDFNCS>* settings;
+ std::vector<MDFNCS>::const_iterator sit;
 
  settings = MDFNI_GetSettings();
 
@@ -624,8 +643,8 @@ static void MakeMednafenArgsStruct(void)
 
  for(sit = settings->begin(); sit != settings->end(); sit++)
  {
-  MDFN_Internal_Args[x].name = strdup(sit->second.name);
-  MDFN_Internal_Args[x].description = sit->second.desc->description ? _(sit->second.desc->description) : NULL;
+  MDFN_Internal_Args[x].name = strdup(sit->name);
+  MDFN_Internal_Args[x].description = sit->desc->description ? _(sit->desc->description) : NULL;
   MDFN_Internal_Args[x].var = NULL;
   MDFN_Internal_Args[x].subs = (void *)HokeyPokeyFallDown;
   MDFN_Internal_Args[x].substype = SUBSTYPE_FUNCTION;
@@ -638,6 +657,7 @@ static void MakeMednafenArgsStruct(void)
 
 static int netconnect = 0;
 static char* loadcd = NULL;	// Deprecated
+static int which_medium = -2;
 
 static char * force_module_arg = NULL;
 static int DoArgs(int argc, char *argv[], char **filename)
@@ -659,6 +679,8 @@ static int DoArgs(int argc, char *argv[], char **filename)
 	 // -loadcd is deprecated and only still supported because it's been around for yeaaaars.
 	 { "loadcd", NULL/*_("Load and boot a CD for the specified system.")*/, 0, &loadcd, SUBSTYPE_STRING_ALLOC },
 
+	 { "which_medium", _("Start with specified disk/CD(numbered from 0) inserted."), 0, &which_medium, SUBSTYPE_INTEGER },
+
 	 { "force_module", _("Force usage of specified emulation module."), 0, &force_module_arg, SUBSTYPE_STRING_ALLOC },
 
 	 { "soundrecord", _("Record sound output to the specified filename in the MS WAV format."), 0,&soundrecfn, SUBSTYPE_STRING_ALLOC },
@@ -784,7 +806,7 @@ static int LoadGame(const char *force_module, const char *path)
 	CurGame = tmp;
 	InitGameInput(tmp);
 	InitCommandInput(tmp);
-	RMDUI_Init(tmp);
+	RMDUI_Init(tmp, which_medium);
 
         RefreshThrottleFPS(1);
 
@@ -811,7 +833,7 @@ static int LoadGame(const char *force_module, const char *path)
 	 MDFNI_LoadState(NULL, "mca");
 
 	if(netconnect)
-	 MDFND_NetworkConnect();
+	 MDFNI_NetplayConnect();
 
 	ers.SetEmuClock(CurGame->MasterClock >> 32);
 
@@ -875,7 +897,7 @@ int CloseGame(void)
 	if(MDFN_GetSettingB("autosave"))
 	 MDFNI_SaveState(NULL, "mca", NULL, NULL, NULL);
 
-	MDFND_NetworkClose();
+	MDFNI_NetplayDisconnect();
 
 	Debugger_Kill();
 
@@ -900,7 +922,7 @@ void MainRequestExit(void)
  NeedExitNow = 1;
 }
 
-bool MainExitPending(void)	// Called from netplay code, so we can break out of blocking loops after receiving a signal.
+bool MDFND_CheckNeedExit(void)	// Called from netplay code, so we can break out of blocking loops after receiving a signal.
 {
  return (bool)NeedExitNow;
 }
@@ -951,7 +973,7 @@ int GameLoop(void *arg)
 	{
          int16 *sound;
          int32 ssize;
-         int fskip;
+         bool fskip;
         
 	 /* If we requested a new video mode, wait until it's set before calling the emulation code again.
 	 */
@@ -966,20 +988,18 @@ int GameLoop(void *arg)
 
 	 if(MDFNDnetplay && !(NoWaiting & 0x2))	// TODO: Hacky, clean up.
 	  ers.SetETtoRT();
-
+	 //
+	 //
 	 fskip = ers.NeedFrameSkip();
-	
-	 if(!MDFN_GetSettingB("video.frameskip"))
-	  fskip = 0;
+	 fskip &= MDFN_GetSettingB("video.frameskip");
+	 fskip &= !(pending_ssnapshot || pending_snapshot || pending_save_state || pending_save_movie || NeedFrameAdvance);
+	 fskip |= (bool)NoWaiting;
 
-	 if(pending_ssnapshot || pending_snapshot || pending_save_state || pending_save_movie || NeedFrameAdvance)
-	  fskip = 0;
-
- 	 NeedFrameAdvance = 0;
-
-         if(NoWaiting)
-	  fskip = 1;
+	 //printf("fskip %d; NeedFrameAdvance=%d\n", fskip, NeedFrameAdvance);
 
+	 NeedFrameAdvance = false;
+	 //
+	 //
 	 SoftFB[SoftFB_BackBuffer].lw[0] = ~0;
 
 	 //
@@ -1046,12 +1066,12 @@ int GameLoop(void *arg)
 
 	  do
 	  {
- 	   if(fskip && GameLoopPaused)
+ 	   if(fskip && ((InFrameAdvance && !NeedFrameAdvance) || GameLoopPaused))
 	   {
-	    // If this frame was skipped, and the game loop is paused(IE cheat interface is active), just blit the previous "successful" frame so the cheat
-	    // interface actually gets drawn.
+	    // If this frame was skipped, and the game loop is paused(IE cheat interface is active) or we're in frame advance, just blit the last
+	    // drawn, non-skipped frame so the OSD elements actually get drawn.
 	    //
-	    // Needless to say, do not do "SoftFB_BackBuffer ^= 1;" here.
+	    // Needless to say, do not allow do_flip to be set to true here.
 	    //
 	    // Possible problems with this kludgery:
 	    //	Will fail spectacularly if there is no previous successful frame.  BOOOOOOM.  (But there always should be, especially since we initialize some
@@ -1064,6 +1084,8 @@ int GameLoop(void *arg)
 
 	   FPS_UpdateCalc();
 
+	   Netplay_GT_CheckPendingLine();
+
            if((InFrameAdvance && !NeedFrameAdvance) || GameLoopPaused)
 	   {
             if(ssize)
@@ -1141,6 +1163,10 @@ static volatile int gte_read = 0;
 static volatile int gte_write = 0;
 
 /* This function may also be called by the main thread if a game is not loaded. */
+/*
+ This function may be called from MDFND_MidSync(), so make sure that it doesn't call directly nor indirectly
+ any MDFNI_* functions that shouldn't be called.
+*/
 static void GameThread_HandleEvents(void)
 {
  SDL_Event gtevents_temp[gtevents_size];
@@ -1177,8 +1203,6 @@ static void GameThread_HandleEvents(void)
 
   if(Debugger_IsActive())
    Debugger_GT_Event(event);
-
-  NetplayEventHook_GT(event);
  }
 }
 
@@ -1691,6 +1715,10 @@ int main(int argc, char *argv[])
 #endif
 	char *needie = NULL;
 
+        // Place before calls to SDL_Init()
+	putenv(strdup("SDL_DISABLE_LOCK_KEYS=1"));
+        //
+
 	MDFNDHaveFocus = false;
 
 	DrBaseDirectory=GetBaseDirectory();
@@ -1842,7 +1870,7 @@ for(int zgi = 1; zgi < argc; zgi++)// start game load test loop
 	  SoftFB[i].surface->Fill(0, 0, 0, 0);
 
 	  //
-	  // Debugger step mode, and cheat interface, rely on the previous backbuffer being valid in certain situations.  Initialize some stuff here so that
+	  // Debugger step mode, cheat interface, and frame advance mode rely on the previous backbuffer being valid in certain situations.  Initialize some stuff here so that
 	  // reliance will still work even immediately after startup.
 	  SoftFB[i].rect.w = std::min<int32>(16, SoftFB[i].surface->w);
 	  SoftFB[i].rect.h = std::min<int32>(16, SoftFB[i].surface->h);
@@ -1946,57 +1974,59 @@ for(int zgi = 1; zgi < argc; zgi++)// start game load test loop
 }
 
 
-
 static uint32 last_btime = 0;
-static void UpdateSoundSync(int16 *Buffer, int Count)
+static void UpdateSoundSync(int16 *Buffer, uint32 Count)
 {
  if(Count)
  {
   if(ffnosound && CurGameSpeed != 1)
   {
-   for(int x = 0; x < Count * CurGame->soundchan; x++)
+   for(uint32 x = 0; x < Count * CurGame->soundchan; x++)
     Buffer[x] = 0;
   }
-  int32 max = Sound_CanWrite();
-  if(Count > max)
+  //
+  //
+  //
+  const uint32 cw = Sound_CanWrite();
+  bool NeedETtoRT = (Count >= (cw * 0.95));
+
+  if(NoWaiting && Count > cw)
   {
-   if(NoWaiting) 
-   {
-    //printf("NW C to M; count=%d, max=%d\n", Count, max);
-    Count = max;
-   }
+   //printf("NW C to M; count=%d, max=%d\n", Count, max);
+   Count = cw;
   }
-  if(Count >= (max * 0.95))
+  else if(MDFNDnetplay)
   {
-   ers.SetETtoRT();
+   //
+   // Cheap code to fix sound buffer underruns due to accumulation of time error during netplay.
+   //
+   uint32 dw = 0;
+
+   if(cw >= (Count * 7 / 4)) // || cw >= Sound_BufferSize())
+    dw = cw - std::min<uint32>(cw, Count);
+
+   if(dw)
+   {
+    int16 zbuf[128 * 2];	// *2 for stereo case.
+
+    //printf("DW: %u\n", dw);
+
+    memset(zbuf, 0, sizeof(zbuf));
+
+    while(dw != 0)
+    {
+     uint32 wti = std::min<int>(128, dw);
+     Sound_Write(zbuf, wti);
+     dw -= wti;
+    }
+    NeedETtoRT = true;
+   }
   }
 
   Sound_Write(Buffer, Count);
 
-  //printf("%u\n", Sound_CanWrite());
-
-  //
-  // Cheap code to fix sound buffer underruns due to accumulation of time error during netplay.
-  //
-  if(MDFNDnetplay)
-  {
-   int cw = Sound_CanWrite();
-
-   if(cw >= Count * 1.00)
-   {
-    int16 zbuf[128 * 2];	// *2 for stereo case.
-
-    //printf("SNOO: %d %d\n", cw, Count);
-    memset(zbuf, 0, sizeof(zbuf));
-
-    while(cw > 0)
-    {
-     Sound_Write(zbuf, std::min<int>(128, cw));
-     cw -= 128;
-    }
-    ers.SetETtoRT();
-   }
-  }
+  if(NeedETtoRT)
+   ers.SetETtoRT();
  }
  else
  {
@@ -2013,7 +2043,7 @@ void MDFND_MidSync(const EmulateSpecStruct *espec)
 
  UpdateSoundSync(espec->SoundBuf + (espec->SoundBufSizeALMS * CurGame->soundchan), espec->SoundBufSize - espec->SoundBufSizeALMS);
 
- // TODO(once we can ensure it's safe): GameThread_HandleEvents();
+ GameThread_HandleEvents(); // Should be safe, but be careful about future changes.
  MDFND_UpdateInput(true, false);
 }
 
