@@ -25,9 +25,6 @@
 #include <mednafen/resampler/resampler.h>
 #include <mednafen/cheat_formats/snes.h>
 
-#include <vector>
-#include <memory>
-
 #include "src/base.hpp"
 
 extern MDFNGI EmulatedSNES;
@@ -99,9 +96,9 @@ static void BuildColorMap(MDFN_PixelFormat &format, uint8* CustomColorMap)
  }
 }
 
-static CustomPalette_Spec CPInfo[] =
+static const CustomPalette_Spec CPInfo[] =
 {
- { gettext_noop("SNES 15-bit RGB colormap"), NULL, { 32768, 0 } },
+ { gettext_noop("SNES 15-bit BGR"), NULL, { 32768, 0 } },
 
  { NULL, NULL }
 };
