@@ -2,7 +2,7 @@
 /* Mednafen Sega Saturn Emulation Module                                      */
 /******************************************************************************/
 /* ss.cpp - Saturn Core Emulation and Support Functions
-**  Copyright (C) 2015-2016 Mednafen Team
+**  Copyright (C) 2015-2017 Mednafen Team
 **
 ** This program is free software; you can redistribute it and/or
 ** modify it under the terms of the GNU General Public License
@@ -131,7 +131,16 @@ static INLINE void BusRW(uint32 A, T& V, const bool BurstHax, int32* SH2DMAHax)
   if(!BurstHax)
   {
    if(!SH2DMAHax)
-    SH7095_mem_timestamp += IsWrite ? 4 : 7;
+   {
+    if(IsWrite)
+    {
+     SH7095_mem_timestamp = (SH7095_mem_timestamp + 4) &~ 3;
+    }
+    else
+    {
+     SH7095_mem_timestamp += 7;
+    }
+   }
    else
     *SH2DMAHax -= IsWrite ? 3 : 6;
   }
@@ -269,7 +278,7 @@ static INLINE void BusRW(uint32 A, T& V, const bool BurstHax, int32* SH2DMAHax)
    else
     *SH2DMAHax -= 8;
 
-   //printf("FT FRT %zu %08x %04x %d\n", sizeof(T), A, V, SMPC_IsSlaveOn());
+   //printf("FT FRT%08x %zu %08x %04x %d %d\n", A, sizeof(T), A, V, SMPC_IsSlaveOn(), SH7095_mem_timestamp);
 
    if(IsWrite)
    {
@@ -492,6 +501,8 @@ static MDFN_COLD void InitEvents(void)
 
  events[SS_EVENT_SOUND].event_handler = SOUND_Update;
 
+ events[SS_EVENT_CART].event_handler = CART_GetEventHandler();
+
  events[SS_EVENT_MIDSYNC].event_handler = MidSync;
  events[SS_EVENT_MIDSYNC].event_time = SS_EVENT_DISABLED_TS;
 }
@@ -703,6 +714,17 @@ void SS_Reset(bool powering_up)
 static EmulateSpecStruct* espec;
 static bool AllowMidSync;
 static int32 cur_clock_div;
+
+static int64 UpdateInputLastBigTS;
+static INLINE void UpdateSMPCInput(const sscpu_timestamp_t timestamp)
+{
+ int32 elapsed_time = (((int64)timestamp * cur_clock_div * 1000 * 1000) - UpdateInputLastBigTS) / (EmulatedSS.MasterClock / MDFN_MASTERCLOCK_FIXED(1));
+
+ UpdateInputLastBigTS += (int64)elapsed_time * (EmulatedSS.MasterClock / MDFN_MASTERCLOCK_FIXED(1));
+
+ SMPC_UpdateInput(elapsed_time);
+}
+
 static sscpu_timestamp_t MidSync(const sscpu_timestamp_t timestamp)
 {
  if(AllowMidSync)
@@ -718,12 +740,15 @@ static sscpu_timestamp_t MidSync(const sscpu_timestamp_t timestamp)
    espec->MasterCycles = timestamp * cur_clock_div;
   }
   //printf("%d\n", espec->SoundBufSize);
+
+  SMPC_UpdateOutput();
   //
   //
   MDFN_MidSync(espec);
   //
   //
-  SMPC_UpdateInput();
+  UpdateSMPCInput(timestamp);
+
   AllowMidSync = false;
  }
 
@@ -739,9 +764,10 @@ static void Emulate(EmulateSpecStruct* espec_arg)
  MDFNGameInfo->mouse_sensitivity = MDFN_GetSettingF("ss.input.mouse_sensitivity");
 
  cur_clock_div = SMPC_StartFrame(espec);
- SMPC_UpdateInput();
+ UpdateSMPCInput(0);
  VDP2::StartFrame(espec, cur_clock_div == 61);
  SOUND_StartFrame(espec->SoundRate / espec->soundmultiplier, MDFN_GetSettingUI("ss.scsp.resamp_quality"));
+ CART_SetCPUClock(EmulatedSS.MasterClock / MDFN_MASTERCLOCK_FIXED(1), cur_clock_div);
  espec->SoundBufSize = 0;
  espec->MasterCycles = 0;
  espec->soundmultiplier = 1;
@@ -760,6 +786,8 @@ static void Emulate(EmulateSpecStruct* espec_arg)
 
  ForceEventUpdates(end_ts);
  //
+ SMPC_EndFrame(espec, end_ts);
+ //
  //
  //
  RebaseTS(end_ts);
@@ -770,6 +798,9 @@ static void Emulate(EmulateSpecStruct* espec_arg)
  VDP2::AdjustTS(-end_ts);
  SMPC_ResetTS();
  SCU_AdjustTS(-end_ts);
+ CART_AdjustTS(-end_ts);
+
+ UpdateInputLastBigTS -= (int64)end_ts * cur_clock_div * 1000 * 1000;
 
  if(!(SH7095_mem_timestamp & 0x40000000))	// or maybe >= 0 instead?
   SH7095_mem_timestamp -= end_ts;
@@ -787,6 +818,10 @@ static void Emulate(EmulateSpecStruct* espec_arg)
  //
  //
  //
+ SMPC_UpdateOutput();
+ //
+ //
+ //
  if(BackupRAM_Dirty)
  {
   BackupRAM_SaveDelay = (int64)3 * (EmulatedSS.MasterClock / MDFN_MASTERCLOCK_FIXED(1));	// 3 second delay
@@ -1072,6 +1107,8 @@ static void MDFN_COLD InitCommon(const unsigned cart_type, const unsigned smpc_a
    { CART_EXTRAM_4M, _("4MiB Extended RAM") },
    { CART_KOF95, _("King of Fighters '95 ROM") },
    { CART_ULTRAMAN, _("Ultraman ROM") },
+   { CART_CS1RAM_16M, _("16MiB CS1 RAM") },
+   { CART_NLMODEM, _("Netlink Modem") },
    { CART_MDFN_DEBUG, _("Mednafen Debug") }, 
   };
   const char* cn = _("Unknown");
@@ -1110,7 +1147,6 @@ static void MDFN_COLD InitCommon(const unsigned cart_type, const unsigned smpc_a
  MDFNMP_RegSearchable(0x06000000, sizeof(WorkRAMH));
 
  CART_Init(cart_type);
-
  //
  //
  //
@@ -1188,13 +1224,53 @@ static void MDFN_COLD InitCommon(const unsigned cart_type, const unsigned smpc_a
  SMPC_Init(smpc_area, MasterClock);
  VDP1::Init();
  VDP2::Init(PAL);
- VDP2::SetGetVideoParams(&EmulatedSS, MDFN_GetSettingB("ss.correct_aspect"), sls, sle, MDFN_GetSettingB("ss.h_overscan"), MDFN_GetSettingB("ss.h_blend"));
  CDB_Init();
  SOUND_Init();
 
  InitEvents();
+ UpdateInputLastBigTS = 0;
 
  DBG_Init();
+ //
+ //
+ //
+ MDFN_printf("\n");
+ {
+  const bool correct_aspect = MDFN_GetSettingB("ss.correct_aspect");
+  const bool h_overscan = MDFN_GetSettingB("ss.h_overscan");
+  const bool h_blend = MDFN_GetSettingB("ss.h_blend");
+
+  MDFN_printf(_("Displayed scanlines: [%u,%u]\n"), sls, sle);
+  MDFN_printf(_("Correct Aspect Ratio: %s\n"), correct_aspect ? _("Enabled") : _("Disabled"));
+  MDFN_printf(_("Show H Overscan: %s\n"), h_overscan ? _("Enabled") : _("Disabled"));
+  MDFN_printf(_("H Blend: %s\n"), h_blend ? _("Enabled") : _("Disabled"));
+
+  VDP2::SetGetVideoParams(&EmulatedSS, correct_aspect, sls, sle, h_overscan, h_blend);
+ }
+
+ MDFN_printf("\n");
+ for(unsigned sp = 0; sp < 2; sp++)
+ {
+  char buf[64];
+  bool sv;
+
+  trio_snprintf(buf, sizeof(buf), "ss.input.sport%u.multitap", sp + 1);
+  sv = MDFN_GetSettingB(buf);
+  SMPC_SetMultitap(sp, sv);
+
+  MDFN_printf(_("Multitap on Saturn Port %u: %s\n"), sp + 1, sv ? _("Enabled") : _("Disabled"));
+ }
+
+ for(unsigned vp = 0; vp < 12; vp++)
+ {
+  char buf[64];
+  uint32 sv;
+
+  trio_snprintf(buf, sizeof(buf), "ss.input.port%u.gun_chairs", vp + 1);
+  sv = MDFN_GetSettingUI(buf);
+  SMPC_SetCrosshairsColor(vp, sv);  
+ }
+
  //
  //
  //
@@ -1298,6 +1374,7 @@ static MDFN_COLD void Load(MDFNFILE* fp)
   if(MDFN_GetSettingS("ss.dbg_exe_cdpath") != "")
   {
    RMD_Drive dr;
+   RMD_DriveDefaults drdef;
 
    dr.Name = std::string("Virtual CD Drive");
    dr.PossibleStates.push_back(RMD_State({"Tray Open", false, false, true}));
@@ -1306,7 +1383,12 @@ static MDFN_COLD void Load(MDFNFILE* fp)
    dr.CompatibleMedia.push_back(0);
    dr.MediaMtoPDelay = 2000;
 
+   drdef.State = 2; // Tray Closed
+   drdef.Media = 0;
+   drdef.Orientation = 0;
+
    MDFNGameInfo->RMD->Drives.push_back(dr);
+   MDFNGameInfo->RMD->DrivesDefaults.push_back(drdef);
    MDFNGameInfo->RMD->MediaTypes.push_back(RMD_MediaType({"CD"}));
    MDFNGameInfo->RMD->Media.push_back(RMD_Media({"Test CD", 0}));
 
@@ -1440,7 +1522,7 @@ static MDFN_COLD void LoadCD(std::vector<CDIF *>* CDInterfaces)
   if((cart_type = MDFN_GetSettingI("ss.cart")) == CART__RESERVED)
   {
    cart_type = CART_BACKUP_MEM;
-   DB_LookupCartDB(sgid, &cart_type);
+   DB_LookupCartDB(sgid, fd_id, &cart_type);
   }
 
   if(MDFN_GetSettingB("ss.cd_sanity"))
@@ -1561,9 +1643,40 @@ static MDFN_COLD void LoadRTC(void)
  SMPC_LoadNV(&sds);
 }
 
-static void StateAction(StateMem *sm, const unsigned load, const bool data_only)
+static MDFN_COLD void StateAction(StateMem* sm, const unsigned load, const bool data_only)
 {
+ SFORMAT StateRegs[] = 
+ {
+  // TODO: Events, or recalc?
 
+  SFARRAY16(WorkRAML, sizeof(WorkRAML) / sizeof(WorkRAML[0])),
+  SFARRAY16(WorkRAMH, sizeof(WorkRAMH) / sizeof(WorkRAMH[0])),
+  SFARRAY(BackupRAM, sizeof(BackupRAM) / sizeof(BackupRAM[0])),
+
+  SFEND
+ };
+
+ MDFNSS_StateAction(sm, load, data_only, StateRegs, "MAIN");
+
+ if(load)
+ {
+  BackupRAM_Dirty = true;
+ }
+
+/*
+ CPU[0].StateAction(sm, load, data_only, "SH2-M");
+ CPU[1].StateAction(sm, load, data_only, "SH2-S");
+ SCU_StateAction(sm, load, data_only);
+*/
+ SMPC_StateAction(sm, load, data_only);
+/*
+ CDB_StateAction(sm, load, data_only);
+ VDP1::StateAction(sm, load, data_only);
+ VDP2_StateAction(sm, load, data_only);
+*/
+
+ SOUND_StateAction(sm, load, data_only);
+ CART_StateAction(sm, load, data_only);
 }
 
 static MDFN_COLD bool SetMedia(uint32 drive_idx, uint32 state_idx, uint32 media_idx, uint32 orientation_idx)
@@ -1639,14 +1752,16 @@ static const MDFNSetting_EnumList Cart_List[] =
  { "backup", CART_BACKUP_MEM, gettext_noop("Backup Memory(512KiB)") },
  { "extram1", CART_EXTRAM_1M, gettext_noop("1MiB Extended RAM") },
  { "extram4", CART_EXTRAM_4M, gettext_noop("4MiB Extended RAM") },
+ { "cs1ram16", CART_CS1RAM_16M, gettext_noop("16MiB RAM mapped in A-bus CS1") },
+// { "nlmodem", CART_NLMODEM, gettext_noop("NetLink Modem") },
 
  { NULL, 0 },
 };
 
-static MDFNSetting SSSettings[] =
+static const MDFNSetting SSSettings[] =
 {
- { "ss.bios_jp", MDFNSF_EMU_STATE, gettext_noop("Path to the Japan ROM BIOS"), gettext_noop(""), MDFNST_STRING, "sega_101.bin" },
- { "ss.bios_na_eu", MDFNSF_EMU_STATE, gettext_noop("Path to the North America and Europe ROM BIOS"), gettext_noop(""), MDFNST_STRING, "mpr-17933.bin" },
+ { "ss.bios_jp", MDFNSF_EMU_STATE, gettext_noop("Path to the Japan ROM BIOS"), NULL, MDFNST_STRING, "sega_101.bin" },
+ { "ss.bios_na_eu", MDFNSF_EMU_STATE, gettext_noop("Path to the North America and Europe ROM BIOS"), NULL, MDFNST_STRING, "mpr-17933.bin" },
 
  { "ss.scsp.resamp_quality", MDFNSF_NOFLAGS, gettext_noop("SCSP output resampler quality."),
 	gettext_noop("0 is lowest quality and CPU usage, 10 is highest quality and CPU usage.  The resampler that this setting refers to is used for converting from 44.1KHz to the sampling rate of the host audio device Mednafen is using.  Changing Mednafen's output rate, via the \"sound.rate\" setting, to \"44100\" may bypass the resampler, which can decrease CPU usage by Mednafen, and can increase or decrease audio quality, depending on various operating system and hardware factors."), MDFNST_UINT, "4", "0", "10" },
@@ -1655,6 +1770,21 @@ static MDFNSetting SSSettings[] =
  { "ss.region_default", MDFNSF_EMU_STATE | MDFNSF_UNTRUSTED_SAFE, gettext_noop("Default region to use."), gettext_noop("Used if region autodetection fails or is disabled."), MDFNST_ENUM, "jp", NULL, NULL, NULL, NULL, Region_List },
 
  { "ss.input.mouse_sensitivity", MDFNSF_NOFLAGS, gettext_noop("Emulated mouse sensitivity."), NULL, MDFNST_FLOAT, "0.50", NULL, NULL },
+ { "ss.input.sport1.multitap", MDFNSF_EMU_STATE | MDFNSF_UNTRUSTED_SAFE, gettext_noop("Enable multitap on Saturn port 1."), NULL, MDFNST_BOOL, "0", NULL, NULL },
+ { "ss.input.sport2.multitap", MDFNSF_EMU_STATE | MDFNSF_UNTRUSTED_SAFE, gettext_noop("Enable multitap on Saturn port 2."), NULL, MDFNST_BOOL, "0", NULL, NULL },
+
+ { "ss.input.port1.gun_chairs",  MDFNSF_NOFLAGS, gettext_noop("Crosshairs color for lightgun on virtual port 1."),  gettext_noop("A value of 0x1000000 disables crosshair drawing."), MDFNST_UINT, "0xFF0000", "0x000000", "0x1000000" },
+ { "ss.input.port2.gun_chairs",  MDFNSF_NOFLAGS, gettext_noop("Crosshairs color for lightgun on virtual port 2."),  gettext_noop("A value of 0x1000000 disables crosshair drawing."), MDFNST_UINT, "0x00FF00", "0x000000", "0x1000000" },
+ { "ss.input.port3.gun_chairs",  MDFNSF_NOFLAGS, gettext_noop("Crosshairs color for lightgun on virtual port 3."),  gettext_noop("A value of 0x1000000 disables crosshair drawing."), MDFNST_UINT, "0xFF00FF", "0x000000", "0x1000000" },
+ { "ss.input.port4.gun_chairs",  MDFNSF_NOFLAGS, gettext_noop("Crosshairs color for lightgun on virtual port 4."),  gettext_noop("A value of 0x1000000 disables crosshair drawing."), MDFNST_UINT, "0xFF8000", "0x000000", "0x1000000" },
+ { "ss.input.port5.gun_chairs",  MDFNSF_NOFLAGS, gettext_noop("Crosshairs color for lightgun on virtual port 5."),  gettext_noop("A value of 0x1000000 disables crosshair drawing."), MDFNST_UINT, "0xFFFF00", "0x000000", "0x1000000" },
+ { "ss.input.port6.gun_chairs",  MDFNSF_NOFLAGS, gettext_noop("Crosshairs color for lightgun on virtual port 6."),  gettext_noop("A value of 0x1000000 disables crosshair drawing."), MDFNST_UINT, "0x00FFFF", "0x000000", "0x1000000" },
+ { "ss.input.port7.gun_chairs",  MDFNSF_NOFLAGS, gettext_noop("Crosshairs color for lightgun on virtual port 7."),  gettext_noop("A value of 0x1000000 disables crosshair drawing."), MDFNST_UINT, "0x0080FF", "0x000000", "0x1000000" },
+ { "ss.input.port8.gun_chairs",  MDFNSF_NOFLAGS, gettext_noop("Crosshairs color for lightgun on virtual port 8."),  gettext_noop("A value of 0x1000000 disables crosshair drawing."), MDFNST_UINT, "0x8000FF", "0x000000", "0x1000000" },
+ { "ss.input.port9.gun_chairs",  MDFNSF_NOFLAGS, gettext_noop("Crosshairs color for lightgun on virtual port 9."),  gettext_noop("A value of 0x1000000 disables crosshair drawing."), MDFNST_UINT, "0xFF80FF", "0x000000", "0x1000000" },
+ { "ss.input.port10.gun_chairs", MDFNSF_NOFLAGS, gettext_noop("Crosshairs color for lightgun on virtual port 10."), gettext_noop("A value of 0x1000000 disables crosshair drawing."), MDFNST_UINT, "0x00FF80", "0x000000", "0x1000000" },
+ { "ss.input.port11.gun_chairs", MDFNSF_NOFLAGS, gettext_noop("Crosshairs color for lightgun on virtual port 11."), gettext_noop("A value of 0x1000000 disables crosshair drawing."), MDFNST_UINT, "0x8080FF", "0x000000", "0x1000000" },
+ { "ss.input.port12.gun_chairs", MDFNSF_NOFLAGS, gettext_noop("Crosshairs color for lightgun on virtual port 12."), gettext_noop("A value of 0x1000000 disables crosshair drawing."), MDFNST_UINT, "0xFF8080", "0x000000", "0x1000000" },
 
  { "ss.smpc.autortc", MDFNSF_NOFLAGS, gettext_noop("Automatically set RTC on game load."), gettext_noop("Automatically set the SMPC's emulated Real-Time Clock to the host system's current time and date upon game load."), MDFNST_BOOL, "1" },
  { "ss.smpc.autortc.lang", MDFNSF_NOFLAGS, gettext_noop("BIOS language."), gettext_noop("Also affects language used in some games(e.g. the European release of \"Panzer Dragoon\")."), MDFNST_ENUM, "english", NULL, NULL, NULL, NULL, RTCLang_List },
@@ -1662,6 +1792,8 @@ static MDFNSetting SSSettings[] =
  { "ss.cart", MDFNSF_EMU_STATE | MDFNSF_UNTRUSTED_SAFE, gettext_noop("Expansion cart."), NULL, MDFNST_ENUM, "auto", NULL, NULL, NULL, NULL, Cart_List },
  { "ss.cart.kof95_path", MDFNSF_EMU_STATE, gettext_noop("Path to KoF 95 ROM image."), NULL, MDFNST_STRING, "mpr-18811-mx.ic1" },
  { "ss.cart.ultraman_path", MDFNSF_EMU_STATE, gettext_noop("Path to Ultraman ROM image."), NULL, MDFNST_STRING, "mpr-19367-mx.ic1" },
+
+// { "ss.cart.modem_port", MDFNSF_NOFLAGS, gettext_noop("TCP/IP port to use for modem emulation."), gettext_noop("A value of \"0\" disables network access."), MDFNST_UINT, "4920", "0", "65535" },
  
  { "ss.bios_sanity", MDFNSF_NOFLAGS, gettext_noop("Enable BIOS ROM image sanity checks."), NULL, MDFNST_BOOL, "1" },
 
@@ -1743,7 +1875,7 @@ MDFNGI EmulatedSS =
  false,
  NULL, //StateAction,
  Emulate,
- NULL,
+ SMPC_TransformInput,
  SMPC_SetInput,
  SetMedia,
  DoSimpleCommand,
