@@ -2,7 +2,7 @@
 /* Mednafen Sega Saturn Emulation Module                                      */
 /******************************************************************************/
 /* vdp1.cpp - VDP1 Emulation
-**  Copyright (C) 2015-2016 Mednafen Team
+**  Copyright (C) 2015-2017 Mednafen Team
 **
 ** This program is free software; you can redistribute it and/or
 ** modify it under the terms of the GNU General Public License
@@ -28,6 +28,9 @@
 
 // TODO: Test 1x1 line, polyline, sprite, and polygon.
 
+// TODO: Framebuffer swap/auto drawing start happens a bit too early, should happen near
+//       end of hblank instead of the beginning.
+
 #include "ss.h"
 #include <mednafen/mednafen.h>
 #include <mednafen/FileStream.h>
@@ -160,9 +163,6 @@ void Reset(bool powering_up)
   EWDR = 0;
   EWLR = 0;
   EWRR = 0;
-
-  TVMR = 0;
-  FBCR = 0;
  }
 
  UserClipX0 = 0;
@@ -188,8 +188,14 @@ void Reset(bool powering_up)
  RetCommandAddr = -1;
  DrawingActive = false;
 
+ //
+ // Begin confirmed.
+ TVMR = 0;
+ FBCR = 0;
  PTMR = 0;
  EDSR = 0;
+ // End confirmed.
+ //
 
  memset(&EraseParams, 0, sizeof(EraseParams));
  EraseYCounter = ~0U;
@@ -384,7 +390,12 @@ sscpu_timestamp_t Update(sscpu_timestamp_t timestamp)
  if(CycleCounter > VDP1_UpdateTimingGran)
   CycleCounter = VDP1_UpdateTimingGran;
 
- if(DrawingActive)
+ if(CycleCounter > 0 && SCU_CheckVDP1HaltKludge())
+ {
+  //puts("Kludge");
+  CycleCounter = 0;
+ }
+ else if(DrawingActive)
  {
   while(CycleCounter > 0)
   {
