@@ -2,7 +2,7 @@
 /* Mednafen Sega Saturn Emulation Module                                      */
 /******************************************************************************/
 /* vdp2.cpp - VDP2 Emulation
-**  Copyright (C) 2015-2016 Mednafen Team
+**  Copyright (C) 2015-2017 Mednafen Team
 **
 ** This program is free software; you can redistribute it and/or
 ** modify it under the terms of the GNU General Public License
@@ -51,7 +51,7 @@ static uint16 RawRegs[0x100];	// For debugging
 
 static bool DisplayOn;
 static bool BorderMode;
-static bool ExLatchEnable;
+bool ExLatchEnable;
 static bool ExSyncEnable;
 static bool ExBGEnable;
 static bool DispAreaSelect;
@@ -208,6 +208,9 @@ static uint32 VPhase;
 /*static*/ int32 VCounter;
 static bool InternalVB;
 static bool Odd;
+
+static uint32 CRTLineCounter;
+static bool Clock28M;
 //
 static int SurfInterlaceField;
 //
@@ -302,6 +305,8 @@ static uint32 HPhase;
 
 static uint16 Latched_VCNT, Latched_HCNT;
 static bool HVIsExLatched;
+bool ExLatchIn;
+bool ExLatchPending;
 
 static void LatchHV(void)
 {
@@ -323,15 +328,21 @@ static void LatchHV(void)
   Latched_HCNT = (HCounter + (0x200 - HTimings[HRes & 1][HPHASE__COUNT - 1])) << 1;
  else
   Latched_HCNT = HCounter << 1;
-
- //HVIsLatched = true; // Only on external signal-induced latching?
 }
+
 //
 //
+void GetGunXTranslation(const bool clock28m, float* scale, float* offs)
+{
+ VDP2REND_GetGunXTranslation(clock28m, scale, offs);
+}
+
 void StartFrame(EmulateSpecStruct* espec, const bool clock28m)
 {
+ Clock28M = clock28m;
  //printf("StartFrame: %d\n", SurfInterlaceField);
  VDP2REND_StartFrame(espec, clock28m, SurfInterlaceField);
+ CRTLineCounter = 0;
 }
 
 //
@@ -380,6 +391,10 @@ static INLINE void IncVCounter(const sscpu_timestamp_t event_timestamp)
    InternalVB = true;
    Out_VB = true;
   }
+  else if(VPhase == VPHASE_BOTTOM_BLANKING)
+  {
+   CRTLineCounter = 0x80000000U;
+  }
   else if(VPhase == VPHASE_VSYNC)
   {
    if(InterlaceMode)
@@ -398,7 +413,7 @@ static INLINE void IncVCounter(const sscpu_timestamp_t event_timestamp)
 
  RecalcVRAMPenalty();
 
- SMPC_SetVB(event_timestamp, Out_VB);
+ SMPC_SetVBVS(event_timestamp, Out_VB, VPhase == VPHASE_VSYNC);
 }
 
 static INLINE int32 AddHCounter(const sscpu_timestamp_t event_timestamp, int32 count)
@@ -422,6 +437,13 @@ static INLINE int32 AddHCounter(const sscpu_timestamp_t event_timestamp, int32 c
   //
   if(HPhase == HPHASE_ACTIVE)
   {
+   {
+    const int32 div = Clock28M ? 61 : 65;
+    const int32 coord_adj = 6832 - 80 * div;
+
+    SMPC_LineHook(event_timestamp, CRTLineCounter, div, coord_adj);
+   }
+
    if(VPhase == VPHASE_ACTIVE)
    {
     VDP2Rend_LIB* lib = VDP2REND_GetLIB(VCounter);
@@ -471,10 +493,14 @@ static INLINE int32 AddHCounter(const sscpu_timestamp_t event_timestamp, int32 c
     //printf("%d, 0x%08x(%f) 0x%08x(%f)\n", VCounter, RotParams[0].KAstAccum >> 10, (int32)RotParams[0].DKAst / 1024.0, RotParams[1].KAstAccum >> 10, (int32)RotParams[1].DKAst / 1024.0);
     //printf("DL: %d\n", VCounter);
     lib->vdp1_hires8 = VDP1::GetLine(VCounter, lib->vdp1_line, (HRes & 1) ? 352 : 320, (int32)RotParams[0].XstAccum >> 1, (int32)RotParams[0].YstAccum >> 1, (int32)RotParams[0].DX >> 1, (int32)RotParams[0].DY >> 1); // Always call, has side effects.
-    VDP2REND_DrawLine(InternalVB ? -1 : VCounter, !Odd);
+    VDP2REND_DrawLine(InternalVB ? -1 : VCounter, CRTLineCounter, !Odd);
+    CRTLineCounter++;
    }
    else if(VPhase == VPHASE_TOP_BORDER || VPhase == VPHASE_BOTTOM_BORDER)
-    VDP2REND_DrawLine(-1, !Odd);
+   {
+    VDP2REND_DrawLine(-1, CRTLineCounter, !Odd);
+    CRTLineCounter++;
+   }
   }
   else if(HPhase == HPHASE_HSYNC)
   {
@@ -507,6 +533,17 @@ sscpu_timestamp_t Update(sscpu_timestamp_t timestamp)
  if(tmp < ne)
   ne = tmp;
 
+ //
+ //
+ //
+ if(MDFN_UNLIKELY(ExLatchPending))
+ {
+  LatchHV();
+  HVIsExLatched = true;
+  ExLatchPending = false;
+  //printf("ExLatch: %04x %04x\n", Latched_VCNT, Latched_HCNT);
+ }
+
  return lastts + (ne << 2);
 }
 
@@ -793,6 +830,8 @@ void Init(const bool IsPAL)
 
  SS_SetPhysMemMap(0x05E00000, 0x05EFFFFF, VRAM, 0x80000, true);
 
+ ExLatchIn = false;
+
  VDP2REND_Init(IsPAL);
 }
 
@@ -811,6 +850,9 @@ void Kill(void)
  VDP2REND_Kill();
 }
 
+//
+// TODO: Check reset versus power on values.
+//
 void Reset(bool powering_up)
 {
  DisplayOn = false;
@@ -845,7 +887,19 @@ void Reset(bool powering_up)
  }
  RPTA = 0;
  memset(RotParams, 0, sizeof(RotParams));
+ //
+ //
+ //
+ if(powering_up)
+ {
+  HPhase = 0;
+  HCounter = 0;
 
+  Latched_VCNT = 0;
+  Latched_HCNT = 0;
+  HVIsExLatched = false;
+  ExLatchPending = false;
+ }
  //
  // FIXME(init values), also in VDP2REND.
  if(powering_up)
